"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.build = void 0;
const path_1 = __importDefault(require("path"));
const resolve_from_1 = __importDefault(require("resolve-from"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const semver_1 = require("semver");
const consola_1 = __importDefault(require("consola"));
const build_utils_1 = require("@vercel/build-utils");
const utils_1 = require("./utils");
const typescript_1 = require("./typescript");
async function build(opts) {
    const { files, entrypoint, workPath, config = {}, meta = {} } = opts;
    // ---------------- Debugging context --------------
    consola_1.default.log('Running with @nuxt/vercel-builder version', require('../package.json').version);
    // ----------------- Prepare build -----------------
    utils_1.startStep('Prepare build');
    // Validate entrypoint
    utils_1.validateEntrypoint(entrypoint);
    // Get Nuxt directory
    const entrypointDirname = path_1.default.dirname(entrypoint);
    // Get Nuxt path
    const entrypointPath = path_1.default.join(workPath, entrypointDirname);
    // Get folder where we'll store node_modules
    const modulesPath = path_1.default.join(entrypointPath, 'node_modules');
    // Create a real filesystem
    consola_1.default.log('Downloading files...');
    await build_utils_1.download(files, workPath, meta);
    // Change current working directory to entrypointPath
    process.chdir(entrypointPath);
    consola_1.default.log('Working directory:', process.cwd());
    // Read package.json
    let pkg;
    try {
        pkg = await utils_1.readJSON('package.json');
    }
    catch (e) {
        throw new Error(`Can not read package.json from ${entrypointPath}`);
    }
    // Node version
    const nodeVersion = await build_utils_1.getNodeVersion(entrypointPath, undefined, {}, meta);
    const spawnOpts = build_utils_1.getSpawnOptions(meta, nodeVersion);
    // Prepare TypeScript environment if required.
    const usesTypescript = (pkg.devDependencies && Object.keys(pkg.devDependencies).includes('@nuxt/typescript-build')) || (pkg.dependencies && Object.keys(pkg.dependencies).includes('@nuxt/typescript'));
    const needsTypescriptBuild = utils_1.getNuxtConfigName(entrypointPath) === 'nuxt.config.ts';
    if (usesTypescript) {
        await typescript_1.prepareTypescriptEnvironment({
            pkg, spawnOpts, rootDir: entrypointPath
        });
    }
    // Detect npm (prefer yarn)
    const isYarn = !fs_extra_1.default.existsSync('package-lock.json');
    consola_1.default.log('Using', isYarn ? 'yarn' : 'npm');
    // Write .npmrc
    if (process.env.NPM_AUTH_TOKEN) {
        consola_1.default.log('Found NPM_AUTH_TOKEN in environment, creating .npmrc');
        await fs_extra_1.default.writeFile('.npmrc', `//registry.npmjs.org/:_authToken=${process.env.NPM_AUTH_TOKEN}`);
    }
    // Write .yarnclean
    if (isYarn && !fs_extra_1.default.existsSync('.yarnclean')) {
        await fs_extra_1.default.copyFile(path_1.default.join(__dirname, '.yarnclean'), '.yarnclean');
    }
    // ----------------- Install devDependencies -----------------
    utils_1.startStep('Install devDependencies');
    // Prepare node_modules
    try {
        try {
            if (fs_extra_1.default.existsSync(modulesPath)) {
                await fs_extra_1.default.unlink(modulesPath);
            }
            await fs_extra_1.default.mkdirp('node_modules_dev');
        }
        catch {
            fs_extra_1.default.moveSync(modulesPath, path_1.default.join(entrypointPath, 'node_modules_dev'));
        }
        await fs_extra_1.default.symlink('node_modules_dev', modulesPath);
    }
    catch (e) {
        consola_1.default.log('Error linking/unlinking node_modules_dev.', e, { ...opts, files: null });
    }
    // Install all dependencies
    await build_utils_1.runNpmInstall(entrypointPath, ['--prefer-offline'], { ...spawnOpts, env: { ...spawnOpts.env, NODE_ENV: 'development' } }, meta);
    // ----------------- Pre build -----------------
    if (pkg.scripts && Object.keys(pkg.scripts).includes('now-build')) {
        utils_1.startStep('Pre build');
        await build_utils_1.runPackageJsonScript(entrypointPath, 'now-build', spawnOpts);
    }
    // ----------------- Nuxt build -----------------
    utils_1.startStep('Nuxt build');
    let compiledTypescriptFiles = {};
    if (needsTypescriptBuild) {
        const { tscOptions } = config;
        compiledTypescriptFiles = await typescript_1.compileTypescriptBuildFiles({ rootDir: entrypointPath, spawnOpts, tscOptions });
    }
    // Read nuxt.config.js
    const nuxtConfigName = 'nuxt.config.js';
    const nuxtConfigFile = utils_1.getNuxtConfig(entrypointPath, nuxtConfigName);
    // Read options from nuxt.config.js otherwise set sensible defaults
    const staticDir = (nuxtConfigFile.dir && nuxtConfigFile.dir.static) ? nuxtConfigFile.dir.static : 'static';
    const publicPath = ((nuxtConfigFile.build && nuxtConfigFile.build.publicPath) ? nuxtConfigFile.build.publicPath : '/_nuxt/').replace(/^\//, '');
    const buildDir = nuxtConfigFile.buildDir ? path_1.default.relative(entrypointPath, nuxtConfigFile.buildDir) : '.nuxt';
    const srcDir = nuxtConfigFile.srcDir ? path_1.default.relative(entrypointPath, nuxtConfigFile.srcDir) : '.';
    const lambdaName = nuxtConfigFile.lambdaName ? nuxtConfigFile.lambdaName : 'index';
    await utils_1.exec('nuxt', [
        'build',
        '--standalone',
        '--no-lock',
        `--config-file "${nuxtConfigName}"`
    ], spawnOpts);
    if (config.generateStaticRoutes) {
        await utils_1.exec('nuxt', [
            'generate',
            '--no-build',
            '--no-lock',
            `--config-file "${nuxtConfigName}"`
        ], spawnOpts);
    }
    // ----------------- Install dependencies -----------------
    utils_1.startStep('Install dependencies');
    // Use node_modules_prod
    try {
        try {
            if (fs_extra_1.default.existsSync(modulesPath)) {
                await fs_extra_1.default.unlink(modulesPath);
            }
            await fs_extra_1.default.mkdirp('node_modules_prod');
        }
        catch {
            fs_extra_1.default.moveSync(modulesPath, path_1.default.join(entrypointPath, 'node_modules_prod'));
        }
        await fs_extra_1.default.symlink('node_modules_prod', modulesPath);
    }
    catch (e) {
        consola_1.default.log('Error linking/unlinking node_modules_prod.', e, { ...opts, files: null });
    }
    // Only keep core dependency
    const nuxtDep = utils_1.preparePkgForProd(pkg);
    await fs_extra_1.default.writeJSON('package.json', pkg);
    await build_utils_1.runNpmInstall(entrypointPath, ['--prefer-offline'], spawnOpts, meta);
    // Validate nuxt version
    const nuxtPkg = require(resolve_from_1.default(entrypointPath, `@nuxt/core${nuxtDep.suffix}/package.json`));
    if (!semver_1.gte(nuxtPkg.version, '2.4.0')) {
        throw new Error(`nuxt >= 2.4.0 is required, detected version ${nuxtPkg.version}`);
    }
    if (semver_1.gt(nuxtPkg.version, '3.0.0')) {
        consola_1.default.warn('WARNING: nuxt >= 3.0.0 is not tested against this builder!');
    }
    // Cleanup .npmrc
    if (process.env.NPM_AUTH_TOKEN) {
        await fs_extra_1.default.unlink('.npmrc');
    }
    // ----------------- Collect artifacts -----------------
    utils_1.startStep('Collect artifacts');
    // Static files
    const staticFiles = await build_utils_1.glob('**', path_1.default.join(entrypointPath, srcDir, staticDir));
    // Client dist files
    const clientDistDir = path_1.default.join(entrypointPath, buildDir, 'dist/client');
    const clientDistFiles = await utils_1.globAndPrefix('**', clientDistDir, publicPath);
    // Server dist files
    const serverDistDir = path_1.default.join(entrypointPath, buildDir, 'dist/server');
    const serverDistFiles = await utils_1.globAndPrefix('**', serverDistDir, path_1.default.join(buildDir, 'dist/server'));
    // Generated static files
    const generatedDir = path_1.default.join(entrypointPath, 'dist');
    const generatedPagesFiles = config.generateStaticRoutes ? await utils_1.globAndPrefix('**/*.html', generatedDir, './') : {};
    // node_modules_prod
    const nodeModules = await utils_1.globAndPrefix('**', modulesPath, 'node_modules');
    // Lambdas
    const lambdas = {};
    const launcherPath = path_1.default.join(__dirname, 'launcher.js');
    const launcherSrc = (await fs_extra_1.default.readFile(launcherPath, 'utf8'))
        .replace(/__NUXT_SUFFIX__/g, nuxtDep.suffix)
        .replace(/__NUXT_CONFIG__/g, './' + nuxtConfigName);
    const launcherFiles = {
        'vercel__launcher.js': new build_utils_1.FileBlob({ data: launcherSrc }),
        'vercel__bridge.js': new build_utils_1.FileFsRef({ fsPath: require('@vercel/node-bridge') }),
        [nuxtConfigName]: new build_utils_1.FileFsRef({ fsPath: path_1.default.resolve(entrypointPath, nuxtConfigName) }),
        ...serverDistFiles,
        ...compiledTypescriptFiles,
        ...nodeModules
    };
    // Extra files to be included in lambda
    const serverFiles = [
        ...(Array.isArray(config.includeFiles) ? config.includeFiles : config.includeFiles ? [config.includeFiles] : []),
        ...(Array.isArray(config.serverFiles) ? config.serverFiles : []),
        'package.json'
    ];
    for (const pattern of serverFiles) {
        const files = await build_utils_1.glob(pattern, entrypointPath);
        Object.assign(launcherFiles, files);
    }
    // lambdaName will be titled index, unless specified in nuxt.config.js
    lambdas[lambdaName] = await build_utils_1.createLambda({
        handler: 'vercel__launcher.launcher',
        runtime: nodeVersion.runtime,
        files: launcherFiles,
        environment: {
            NODE_ENV: 'production'
        },
        //
        maxDuration: config.maxDuration,
        memory: config.memory
    });
    // await download(launcherFiles, rootDir)
    utils_1.endStep();
    return {
        output: {
            ...lambdas,
            ...clientDistFiles,
            ...staticFiles,
            ...generatedPagesFiles
        },
        routes: [
            { src: `/${publicPath}.+`, headers: { 'Cache-Control': 'max-age=31557600' } },
            ...Object.keys(staticFiles).map(file => ({ src: `/${file}`, headers: { 'Cache-Control': 'max-age=31557600' } })),
            { handle: 'filesystem' },
            { src: '/(.*)', dest: '/index' }
        ]
    };
}
exports.build = build;
